@@ -1,16 +1,14 @@
-/* $Id: ruby_xml_attr.h 612 2008-11-21 08:01:29Z cfis $ */
+/* $Id: ruby_xml_attr.h 758 2009-01-25 20:36:03Z cfis $ */
 
 /* Please see the LICENSE file for copyright and distribution information */
 
-#ifndef __rxml_ATTR__
-#define __rxml_ATTR__
+#ifndef __RXML_ATTR__
+#define __RXML_ATTR__
 
 extern VALUE cXMLAttr;
 
-void ruby_init_xml_attr(void);
-VALUE rxml_attr_new(xmlAttrPtr xattr);
+void rxml_init_attr(void);
+VALUE rxml_attr_wrap(xmlAttrPtr xattr);
 VALUE rxml_attr_value_get(VALUE self);
 VALUE rxml_attr_value_set(VALUE self, VALUE val);
-void rxml_attr_free(xmlAttrPtr xattr);
-VALUE rxml_attr_wrap(xmlAttrPtr xattr);
 #endif
