@@ -2,12 +2,12 @@
 
 /* Please see the LICENSE file for copyright and distribution information */
 
-#ifndef __rxml_ATTRIBUTES__
-#define __rxml_ATTRIBUTES__
+#ifndef __RXML_ATTRIBUTES__
+#define __RXML_ATTRIBUTES__
 
 extern VALUE cXMLAttributesibutes;
 
-void ruby_init_xml_attributes(void);
+void rxml_init_attributes(void);
 VALUE rxml_attributes_new(xmlNodePtr xnode);
 
 VALUE rxml_attributes_attribute_get(VALUE self, VALUE name);
