@@ -1,13 +1,13 @@
-/* $Id: ruby_xml_document.h 614 2008-11-22 08:04:39Z cfis $ */
+/* $Id: ruby_xml_document.h 758 2009-01-25 20:36:03Z cfis $ */
 
 /* Please see the LICENSE file for copyright and distribution information */
 
-#ifndef __rxml_DOCUMENT__
-#define __rxml_DOCUMENT__
+#ifndef __RXML_DOCUMENT__
+#define __RXML_DOCUMENT__
 
 extern VALUE cXMLDocument;
 
-void ruby_init_xml_document();
+void rxml_init_document();
 
 
 #if defined(_WIN32)
