@@ -1,39 +1,47 @@
 #include "ruby_libxml.h"
-#include "ruby_xml_dtd.h"
-#include "ruby_xml_error.h"
 
 #include <libxml/xmlerror.h>
 
 VALUE eXMLError;
+static ID CALL_METHOD;
 static ID ERROR_HANDLER_ID;
+static ID ON_ERROR_METHOD;
 
 /*
-* Document-class: LibXML::XML::Error
-*
-* The XML::Error class exposes libxml errors as
-* standard Ruby exceptions.  When appropriate,
-* libxml-ruby will raise an exception - for example,
-* when parsing a non-well formed xml document.
-*
-* By default, warnings, errors and fatal errors that
-* libxml generates are printed to STDERR via the
-* XML::Error::VERBOSE_HANDLER proc.
-*
-* To disable this output you can install the quiet handler:
-*
-*   XML::Error.set_handler(&XML::Error::QUITE_HANDLER)
-*
-* Install your own handler:
-*
-*   XML::Error.set_handler do |error|
-*     puts error.to_s
-*   end
-*
-* Or remove all handlers:
-*
-*   XML::Error.reset_handler
-*/
+ * Document-class: LibXML::XML::Error
+ *
+ * The XML::Error class exposes libxml errors as
+ * standard Ruby exceptions.  When appropriate,
+ * libxml-ruby will raise an exception - for example,
+ * when parsing a non-well formed xml document.
+ *
+ * By default, warnings, errors and fatal errors that
+ * libxml generates are printed to STDERR via the
+ * XML::Error::VERBOSE_HANDLER proc.
+ *
+ * To disable this output you can install the quiet handler:
+ *
+ *   XML::Error.set_handler(&XML::Error::QUIET_HANDLER)
+ *
+ * Install your own handler:
+ *
+ *   XML::Error.set_handler do |error|
+ *     puts error.to_s
+ *   end
+ *
+ * Or remove all handlers:
+ *
+ *   XML::Error.reset_handler
+ */
 
+static void rxml_set_handler(VALUE self, VALUE block)
+{
+#ifdef RB_CVAR_SET_4ARGS
+  rb_cvar_set(self, ERROR_HANDLER_ID, block, 0);
+#else
+  rb_cvar_set(self, ERROR_HANDLER_ID, block);
+#endif
+}
 
 /*
  * call-seq:
@@ -43,8 +51,7 @@ static ID ERROR_HANDLER_ID;
  * XML::Error when libxml generates warning, error or fatal
  * error messages.
  */
-static VALUE
-rxml_error_set_handler(VALUE self)
+static VALUE rxml_error_set_handler(VALUE self)
 {
   VALUE block;
 
@@ -54,8 +61,8 @@ rxml_error_set_handler(VALUE self)
   block = rb_block_proc();
 
   /* Embed the block within the Error class to avoid it to be collected.
-     Previous handler will be overwritten if it exists. */
-  rb_cvar_set(self, ERROR_HANDLER_ID, block, 0);
+   Previous handler will be overwritten if it exists. */
+  rxml_set_handler(self, block);
 
   return self;
 }
@@ -65,17 +72,16 @@ rxml_error_set_handler(VALUE self)
  *    Error.reset_error_handler
  *
  * Removes the current error handler. */
-static VALUE
-rxml_error_reset_handler(VALUE self)
+static VALUE rxml_error_reset_handler(VALUE self)
 {
-  rb_cvar_set(self, ERROR_HANDLER_ID, Qnil, 0);
+  rxml_set_handler(self, Qnil);
   return self;
 }
 
-
-static VALUE
-rxml_error_wrap(xmlErrorPtr xerror) {
+VALUE rxml_error_wrap(xmlErrorPtr xerror)
+{
   VALUE result = Qnil;
+
   if (xerror->message)
     result = rb_exc_new2(eXMLError, xerror->message);
   else
@@ -103,45 +109,73 @@ rxml_error_wrap(xmlErrorPtr xerror) {
   rb_iv_set(result, "@int1", INT2NUM(xerror->int1));
   rb_iv_set(result, "@int2", INT2NUM(xerror->int2));
 
-  //rb_define_attr(eXMLError, "ctxt", 1, 0);
-  if (xerror->node) {
-    VALUE node = rxml_node2_wrap(cXMLNode, xerror->node);
+  if (xerror->node)
+  {
+    /* Returning the original node is too dangerous because its 
+       parent document is never returned to Ruby.  So return a 
+       copy of the node, which does not belong to any document,
+       and can free itself when Ruby calls its free method.  Note
+       we just copy the node, and don't bother with the overhead
+       of a recursive query. */
+    xmlNodePtr xNode = xmlCopyNode((const xmlNodePtr)xerror->node, 2);
+    VALUE node = rxml_node_wrap(xNode);
     rb_iv_set(result, "@node", node);
   }
   return result;
 }
 
 /* Hook that receives xml error message */
-static void
-structuredErrorFunc(void *userData, xmlErrorPtr xerror)
+static void structuredErrorFunc(void *userData, xmlErrorPtr xerror)
 {
+  VALUE error = rxml_error_wrap(xerror);
+
   /* Wrap error up as Ruby object and send it off to ruby */
   VALUE block = rb_cvar_get(eXMLError, ERROR_HANDLER_ID);
 
+  /* This next bit of code is a total hack to get around a bug
+     in libxml which causes error handlers on sax handlers to
+     be ignored in favor of the global handler.  In addition,
+     the correct context is also not passed in.  So try to 
+     dig it out. */
+  if (!userData && xerror->ctxt)
+  {
+    xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr) xerror->ctxt;
+    if (ctxt != ctxt->userData)
+      userData = ctxt->userData;
+  }
+
+  /* If the target has an on_error method call it.  This
+   gets around a bug in libxml where a sax's structured
+   error handler is overriden by the global error handler. */
+  if (userData)
+  {  
+    VALUE target = (VALUE) userData;
+    if (!NIL_P(target) && rb_respond_to(target, ON_ERROR_METHOD))
+    {
+      rb_funcall(target, ON_ERROR_METHOD, 1, error);
+    }
+  }
+
+  /* Now call global handler */
   if (block != Qnil)
   {
-    VALUE error = rxml_error_wrap(xerror);
-    rb_funcall(block, rb_intern("call"), 1, error);
+    rb_funcall(block, CALL_METHOD, 1, error);
   }
 }
 
-// Rdoc needs to know
-#ifdef RDOC_NEVER_DEFINED
-  mLibXML = rb_define_module("LibXML");
-  mXML = rb_define_module_under(mLibXML, "XML");
-#endif
-
-void
-rxml_raise(xmlErrorPtr xerror)
+void rxml_raise(xmlErrorPtr xerror)
 {
   /* Wrap error up as Ruby object and send it off to ruby */
   VALUE error = rxml_error_wrap(xerror);
   rb_exc_raise(error);
 }
 
+void rxml_init_error()
+{
+  ON_ERROR_METHOD = rb_intern("on_error");
+  CALL_METHOD = rb_intern("call");
+  ERROR_HANDLER_ID = rb_intern("@@__error_handler_callback__");
 
-void
-ruby_init_xml_error() {
   /* Intercept libxml error handlers */
   xmlSetStructuredErrorFunc(NULL, structuredErrorFunc);
 
@@ -150,10 +184,8 @@ ruby_init_xml_error() {
   rb_define_singleton_method(eXMLError, "set_handler", rxml_error_set_handler, 0);
   rb_define_singleton_method(eXMLError, "reset_handler", rxml_error_reset_handler, 0);
 
-
   /* Ruby callback to receive errors - set it to nil by default. */
-  ERROR_HANDLER_ID = rb_intern("@@__error_handler_callback__");
-  rb_cvar_set(eXMLError, ERROR_HANDLER_ID, Qnil, 0);
+  rxml_set_handler(eXMLError, Qnil);
 
   /* Error attributes */
   rb_define_attr(eXMLError, "level", 1, 0);
@@ -170,10 +202,10 @@ ruby_init_xml_error() {
   rb_define_attr(eXMLError, "node", 1, 0);
 
   /* xml error levels */
-  rb_define_const(eXMLError, "NONE",    INT2NUM(XML_ERR_NONE));
+  rb_define_const(eXMLError, "NONE", INT2NUM(XML_ERR_NONE));
   rb_define_const(eXMLError, "WARNING", INT2NUM(XML_ERR_WARNING));
-  rb_define_const(eXMLError, "ERROR",   INT2NUM(XML_ERR_ERROR));
-  rb_define_const(eXMLError, "FATAL",   INT2NUM(XML_ERR_FATAL));
+  rb_define_const(eXMLError, "ERROR", INT2NUM(XML_ERR_ERROR));
+  rb_define_const(eXMLError, "FATAL", INT2NUM(XML_ERR_FATAL));
 
   /* xml error domains */
   rb_define_const(eXMLError, "NO_ERROR", INT2NUM(XML_FROM_NONE));
@@ -231,7 +263,7 @@ ruby_init_xml_error() {
   rb_define_const(eXMLError, "ENTITYREF_IN_DTD", INT2NUM(XML_ERR_ENTITYREF_IN_DTD));
   rb_define_const(eXMLError, "PEREF_AT_EOF", INT2NUM(XML_ERR_PEREF_AT_EOF));
   rb_define_const(eXMLError, "PEREF_IN_PROLOG", INT2NUM(XML_ERR_PEREF_IN_PROLOG));
-  rb_define_const(eXMLError, "PEREF_IN_EPILOG", INT2NUM(XML_ERR_PEREF_IN_EPILOG));
+  rb_define_const(eXMLError, "PEREF_IN_EPILOG",INT2NUM(XML_ERR_PEREF_IN_EPILOG));
   rb_define_const(eXMLError, "PEREF_IN_INT_SUBSET", INT2NUM(XML_ERR_PEREF_IN_INT_SUBSET));
   rb_define_const(eXMLError, "ENTITYREF_NO_NAME", INT2NUM(XML_ERR_ENTITYREF_NO_NAME));
   rb_define_const(eXMLError, "ENTITYREF_SEMICOL_MISSING", INT2NUM(XML_ERR_ENTITYREF_SEMICOL_MISSING));
@@ -771,7 +803,7 @@ ruby_init_xml_error() {
   rb_define_const(eXMLError, "SCHEMAV_CVC_ELT_5_2_2_2_1", INT2NUM(XML_SCHEMAV_CVC_ELT_5_2_2_2_1));
   rb_define_const(eXMLError, "SCHEMAV_CVC_ELT_5_2_2_2_2", INT2NUM(XML_SCHEMAV_CVC_ELT_5_2_2_2_2));
   rb_define_const(eXMLError, "SCHEMAV_CVC_ELT_6", INT2NUM(XML_SCHEMAV_CVC_ELT_6));
-  rb_define_const(eXMLError, "SCHEMAV_CVC_ELT_7", INT2NUM(XML_SCHEMAV_CVC_ELT_7));
+  rb_define_const(eXMLError, "SCHEMAV_CVC_ELT_7",INT2NUM(XML_SCHEMAV_CVC_ELT_7));
   rb_define_const(eXMLError, "SCHEMAV_CVC_ATTRIBUTE_1", INT2NUM(XML_SCHEMAV_CVC_ATTRIBUTE_1));
   rb_define_const(eXMLError, "SCHEMAV_CVC_ATTRIBUTE_2", INT2NUM(XML_SCHEMAV_CVC_ATTRIBUTE_2));
   rb_define_const(eXMLError, "SCHEMAV_CVC_ATTRIBUTE_3", INT2NUM(XML_SCHEMAV_CVC_ATTRIBUTE_3));
@@ -801,8 +833,10 @@ ruby_init_xml_error() {
   rb_define_const(eXMLError, "XPTR_EXTRA_OBJECTS", INT2NUM(XML_XPTR_EXTRA_OBJECTS));
   rb_define_const(eXMLError, "C14N_CREATE_CTXT", INT2NUM(XML_C14N_CREATE_CTXT));
   rb_define_const(eXMLError, "C14N_REQUIRES_UTF8", INT2NUM(XML_C14N_REQUIRES_UTF8));
-  rb_define_const(eXMLError, "C14N_CREATE_STACK", INT2NUM(XML_C14N_CREATE_STACK));
-  rb_define_const(eXMLError, "C14N_INVALID_NODE", INT2NUM(XML_C14N_INVALID_NODE));
+  rb_define_const(eXMLError, "C14N_CREATE_STACK",
+      INT2NUM(XML_C14N_CREATE_STACK));
+  rb_define_const(eXMLError, "C14N_INVALID_NODE",
+      INT2NUM(XML_C14N_INVALID_NODE));
 #if LIBXML_VERSION >= 20619
   rb_define_const(eXMLError, "C14N_UNKNOW_NODE", INT2NUM(XML_C14N_UNKNOW_NODE));
   rb_define_const(eXMLError, "C14N_RELATIVE_NAMESPACE", INT2NUM(XML_C14N_RELATIVE_NAMESPACE));
@@ -906,7 +940,7 @@ ruby_init_xml_error() {
   rb_define_const(eXMLError, "SCHEMAP_WARN_UNLOCATED_SCHEMA", INT2NUM(XML_SCHEMAP_WARN_UNLOCATED_SCHEMA));
   rb_define_const(eXMLError, "SCHEMAP_WARN_ATTR_REDECL_PROH", INT2NUM(XML_SCHEMAP_WARN_ATTR_REDECL_PROH));
   rb_define_const(eXMLError, "SCHEMAP_WARN_ATTR_POINTLESS_PROH", INT2NUM(XML_SCHEMAP_WARN_ATTR_POINTLESS_PROH));
-#endif  
+#endif
 #if LIBXML_VERSION >= 20623
   rb_define_const(eXMLError, "SCHEMAP_AG_PROPS_CORRECT", INT2NUM(XML_SCHEMAP_AG_PROPS_CORRECT));
   rb_define_const(eXMLError, "SCHEMAP_COS_CT_EXTENDS_1_2", INT2NUM(XML_SCHEMAP_COS_CT_EXTENDS_1_2));
@@ -925,7 +959,7 @@ ruby_init_xml_error() {
   rb_define_const(eXMLError, "CHECK_FOUND_ELEMENT", INT2NUM(XML_CHECK_FOUND_ELEMENT));
   rb_define_const(eXMLError, "CHECK_FOUND_ATTRIBUTE", INT2NUM(XML_CHECK_FOUND_ATTRIBUTE));
   rb_define_const(eXMLError, "CHECK_FOUND_TEXT", INT2NUM(XML_CHECK_FOUND_TEXT));
-  rb_define_const(eXMLError, "CHECK_FOUND_CDATA", INT2NUM(XML_CHECK_FOUND_CDATA));
+  rb_define_const(eXMLError, "CHECK_FOUND_CDATA",INT2NUM(XML_CHECK_FOUND_CDATA));
   rb_define_const(eXMLError, "CHECK_FOUND_ENTITYREF", INT2NUM(XML_CHECK_FOUND_ENTITYREF));
   rb_define_const(eXMLError, "CHECK_FOUND_ENTITY", INT2NUM(XML_CHECK_FOUND_ENTITY));
   rb_define_const(eXMLError, "CHECK_FOUND_PI", INT2NUM(XML_CHECK_FOUND_PI));
