@@ -2,12 +2,13 @@
 
 /* Please see the LICENSE file for copyright and distribution information */
 
-#ifndef __rxml_ERROR__
-#define __rxml_ERROR__
+#ifndef __RXML_ERROR__
+#define __RXML_ERROR__
 
 extern VALUE eXMLError;
 
-void ruby_init_xml_error();
+void rxml_init_error();
+VALUE rxml_error_wrap(xmlErrorPtr xerror);
 void rxml_raise(xmlErrorPtr xerror);
 
 #endif
