@@ -1,14 +1,14 @@
-/* $Id: ruby_xml_parser.h 614 2008-11-22 08:04:39Z cfis $ */
+/* $Id: ruby_xml_parser.h 758 2009-01-25 20:36:03Z cfis $ */
 
 /* Please see the LICENSE file for copyright and distribution information */
 
-#ifndef __rxml_PARSER__
-#define __rxml_PARSER__
+#ifndef __RXML_PARSER__
+#define __RXML_PARSER__
 
 #define MAX_LIBXML_FEATURES_LEN 50
 
 extern VALUE cXMLParser;
 
-void ruby_init_parser();
+void rxml_init_parser();
 
 #endif
