@@ -1,13 +1,13 @@
-/* $Id: ruby_xml_xinclude.h 612 2008-11-21 08:01:29Z cfis $ */
+/* $Id: ruby_xml_xinclude.h 758 2009-01-25 20:36:03Z cfis $ */
 
 /* Please see the LICENSE file for copyright and distribution information */
 
-#ifndef __rxml_XINCLUDE__
-#define __rxml_XINCLUDE__
+#ifndef __RXML_XINCLUDE__
+#define __RXML_XINCLUDE__
 
 extern VALUE cXMLXInclude;
 extern VALUE eXMLXIncludeError;
 
-void ruby_init_xml_xinclude(void);
+void rxml_init_xinclude(void);
 
 #endif
