@@ -2,11 +2,11 @@
 
 /* Please see the LICENSE file for copyright and distribution information */
 
-#ifndef __rxml_XPATH_EXPRESSION__
-#define __rxml_XPATH_EXPRESSION__
+#ifndef __RXML_XPATH_EXPRESSION__
+#define __RXML_XPATH_EXPRESSION__
 
 extern VALUE cXMLXPathExpression;
 
-void ruby_init_xml_xpath_expression(void);
+void rxml_init_xpath_expression(void);
 
 #endif
