@@ -1,20 +1,8 @@
+require 'stringio'
+
 module LibXML
   module XML
     class Node
-      # Register the prefix (i.e., name) of the default namespace to
-      # make xpath searches easier
-      def register_default_namespace(prefix)
-        ns = self.namespace.find do |ns|
-          ns.prefix.nil?
-        end
-
-        if ns
-          NS.new(self, ns.href, prefix)
-        else
-          raise(ArgumentError, "No default namespace was found")
-        end
-      end
-
       # Determines whether this node has attributes
       def attributes?
         attributes.length > 0
@@ -25,38 +13,84 @@ module LibXML
       def clone
         copy(false)
       end
+
+      # call-seq:
+      #    node.inner_xml -> "string"
+      #    node.inner_xml(:indent => true, :encoding => 'UTF-8', :level => 0) -> "string"
+      #
+      # Converts a node's children, to a string representation.  To include
+      # the node, use XML::Node#to_s.  For more information about
+      # the supported options, see XML::Node#to_s.
+      def inner_xml(options = Hash.new)
+        io = StringIO.new
+
+        self.each do |node|
+          io << node.to_s(options)
+        end
+
+        io.string
+      end
       
+      # :call-seq:
+      #   node.dup -> XML::Node
+      #
       # Create a shallow copy of the node.  To create
       # a deep copy call Node#copy(true)
       def dup
         copy(false)
       end
     
-      # Return nodes matching the specified xpath expression.
-      # For more information, please refer to the documentation
-      # for XML::Document#find.
-      def find(xpath, nslist = nil)
+      # call-seq:
+      #   node.context(namespaces=nil) -> XPath::Context
+      #
+      # Returns a new XML::XPathContext for the current node.
+      #
+      # Namespaces is an optional array of XML::NS objects
+      def context(nslist = nil)
         if not self.doc
-          raise(TypeError, "A node must belong to a document before " +
-                           "it can be searched with XPath.")
+          raise(TypeError, "A node must belong to a document before a xpath context can be created")
         end
-      
+
         context = XPath::Context.new(self)
-        context.node = self  
+        context.node = self
         context.register_namespaces_from_node(self)
         context.register_namespaces_from_node(self.doc.root)
         context.register_namespaces(nslist) if nslist
+        context
+      end
 
-        context.find(xpath)
+      # call-seq:
+      #   node.find(namespaces=nil) -> XPath::XPathObject
+      #
+      # Return nodes matching the specified xpath expression.
+      # For more information, please refer to the documentation
+      # for XML::Document#find.
+      #
+      # Namespaces is an optional array of XML::NS objects
+      def find(xpath, nslist = nil)
+        self.context(nslist).find(xpath)
       end
     
+      # call-seq:
+      #   node.find_first(namespaces=nil) -> XML::Node
+      #
       # Return the first node matching the specified xpath expression.
       # For more information, please refer to the documentation
-      # for XML::Node#find.
+      # for the #find method.
       def find_first(xpath, nslist = nil)
         find(xpath, nslist).first
       end
-    
+
+      # call-seq:
+      #   node.namespacess -> XML::Namespaces
+      #   
+      # Returns this node's XML::Namespaces object,
+      # which is used to access the namespaces
+      # associated with this node.
+      def namespaces
+        @namespaces ||= XML::Namespaces.new(self)
+      end
+      
       # -------  Traversal  ----------------
       # Iterates over this node's attributes.
       #
@@ -270,6 +304,82 @@ module LibXML
       alias :children? :first?  
       alias :child :first
       alias :each_child :each
+
+      # --- Deprecated Output ---
+      # :stopdoc:
+      def dump
+        warn('Node#dump is deprecated.  Use Node#to_s instead.')
+        self.to_s
+      end
+
+      # --- Deprecated DOM Manipulation ---
+      def child_add(node)
+        warn('Node#child_add is deprecated.  Use Node#<< instead.')
+        self << node
+      end
+
+      def child=(node)
+        warn('Node#child= is deprecated.  Use Node#<< instead.')
+        self << node
+      end
+
+      # --- Deprecated Namespaces ---
+      def namespace
+        warn('Node#namespace is deprecated.  Use Node#namespaces instead.')
+        self.namespaces.entries
+      end
+
+      def namespace=(value)
+        warn('Node#namespace= is deprecated.  Use Node#namespaces.namespace= instead.')
+        self.namespaces.namespace = value
+      end
+
+      def namespace_node
+        warn('Node#namespace_node is deprecated.  Use Node#namespaces.namespace instead.')
+        self.namespaces.namespace
+      end
+
+      def ns
+        warn('Node#ns is deprecated.  Use Node#namespaces.namespace instead.')
+        self.namespaces.namespace
+      end
+
+      def ns?
+        warn('Node#ns? is deprecated.  Use !Node#namespaces.namespace.nil? instead.')
+        !self.namespaces.namespace.nil?
+      end
+
+      def ns_def
+        warn('Node#ns_def is deprecated.  Use Node#namespaces.definitions instead.')
+        self.namespaces.definitions
+      end
+
+      def ns_def?
+        warn('Node#ns_def? is deprecated.  Use !Node#namespaces.definitions.nil? instead.')
+        !self.namespaces.definitions.nil?
+      end
+
+      def base
+        warn('Node#base is deprecated.  Use Node#base_uri.')
+        self.base_uri
+      end
+
+      def base=(value)
+        warn('Node#base= is deprecated.  Use Node#base_uri=.')
+        self.base_uri = value
+      end
+
+      def search_ns(prefix)
+        warn('Node#search_ns is deprecated.  Use Node#namespaces.find_by_prefix instead.')
+        self.namespaces.find_by_prefix(prefix)
+      end
+
+      def search_href(href)
+        warn('Node#search_href is deprecated.  Use Node#namespaces.find_by_href instead.')
+        self.namespaces.find_by_href(href)
+      end
+
+      # :startdoc:
     end
   end
-end
+end
\ No newline at end of file
