@@ -59,7 +59,7 @@ class AttributesTest < Test::Unit::TestCase
 
     attr = attributes.get_attribute('href')
     assert_instance_of(XML::Attr, attr)
-    assert_instance_of(XML::NS, attr.ns)
+    assert_instance_of(XML::Namespace, attr.ns)
     assert_equal('xlink', attr.ns.prefix)
     assert_equal('http://www.w3.org/1999/xlink', attr.ns.href)
 
@@ -122,9 +122,12 @@ class AttributesTest < Test::Unit::TestCase
     node = @doc.find_first('/city:CityModel/city:cityMember')
     assert_equal(5, node.attributes.length)
 
-    node.attributes.entries.each do |attr|
+    attrs = Array.new
+    node.attributes.each do |attr|
+      attrs << attr
       attr.remove!
     end
+    assert_equal(5, attrs.length)
     assert_equal(0, node.attributes.length)
   end
 end
\ No newline at end of file
