@@ -1,19 +1,8 @@
-require "xml"
+require 'xml'
+require 'stringio'
 require 'test/unit'
 
-class HtmlParserTest < Test::Unit::TestCase
-  def setup
-    @xp = XML::HTMLParser.new()
-    assert_not_nil(@xp)
-    str = '<html><head><meta name=keywords content=nasty></head><body>Hello<br>World</html>'
-    @xp.string = str
-    assert_equal(str, @xp.string)
-  end
-
-  def teardown()
-    @xp = nil
-  end
-
+class HTMLParserTest < Test::Unit::TestCase
   def html_file
     File.expand_path(File.join(File.dirname(__FILE__), 'model/ruby-lang.html'))
   end
@@ -22,8 +11,62 @@ class HtmlParserTest < Test::Unit::TestCase
   def test_file
     xp = XML::HTMLParser.file(html_file)
     assert_instance_of(XML::HTMLParser, xp)
-    assert_equal(html_file, xp.file)
-    assert_equal(html_file, xp.input.file)
+    doc = xp.parse
+    assert_not_nil(doc)
+  end
+
+  def test_noexistent_file
+    error = assert_raise(XML::Error) do
+      XML::HTMLParser.file('i_dont_exist.xml')
+    end
+
+    assert_equal('Warning: failed to load external entity "i_dont_exist.xml".', error.to_s)
+  end
+
+  def test_nil_file
+    error = assert_raise(TypeError) do
+      XML::HTMLParser.file(nil)
+    end
+
+    assert_equal("can't convert nil into String", error.to_s)
+  end
+
+  def test_io
+    File.open(html_file) do |io|
+      xp = XML::HTMLParser.io(io)
+      assert_instance_of(XML::HTMLParser, xp)
+
+      doc = xp.parse
+      assert_instance_of(XML::Document, doc)
+    end
+  end
+
+  def test_io_gc
+    # Test that the reader keeps a reference
+    # to the io object
+    file = File.open(html_file)
+    parser = XML::HTMLParser.io(file)
+    file = nil
+    GC.start
+    assert(parser.parse)
+  end
+
+  def test_nil_io
+    error = assert_raise(TypeError) do
+      XML::HTMLParser.io(nil)
+    end
+
+    assert_equal("Must pass in an IO object", error.to_s)
+  end
+
+  def test_string_io
+    data = File.read(html_file)
+    io = StringIO.new(data)
+    xp = XML::HTMLParser.io(io)
+    assert_instance_of(XML::HTMLParser, xp)
+
+    doc = xp.parse
+    assert_instance_of(XML::Document, doc)
   end
 
   def test_string
@@ -32,40 +75,30 @@ class HtmlParserTest < Test::Unit::TestCase
 
     assert_instance_of(XML::HTMLParser, xp)
     assert_instance_of(XML::HTMLParser, xp)
-    assert_equal(str, xp.string)
-    assert_equal(str, xp.input.string)
 
     doc = xp.parse
     assert_instance_of(XML::Document, doc)
   end
 
-  def test_io
-    File.open(html_file) do |io|
-      xp = XML::HTMLParser.io(io)
-      assert_instance_of(XML::HTMLParser, xp)
-      assert_equal(io, xp.io)
-      assert_equal(io, xp.input.io)
-
-      doc = xp.parse
-      assert_instance_of(XML::Document, doc)
+  def test_nil_string
+    error = assert_raise(TypeError) do
+      XML::HTMLParser.string(nil)
     end
+
+    assert_equal("wrong argument type nil (expected String)", error.to_s)
   end
 
-  def test_string_io
-    data = File.read(html_file)
-    io = StringIO.new(data)
-    xp = XML::HTMLParser.io(io)
-    assert_instance_of(XML::HTMLParser, xp)
-    assert_equal(io, xp.io)
-    assert_equal(io, xp.input.io)
-
-    doc = xp.parse
-    assert_instance_of(XML::Document, doc)
-  end
-
-  def test_libxml_html_parser_parse
-    doc = @xp.parse
+  def test_parse
+    html = <<-EOS
+      <html>
+        <head>
+          <meta name=keywords content=nasty>
+        </head>
+        <body>Hello<br>World</html>
+   EOS
 
+    parser = XML::HTMLParser.string(html)
+    doc = parser.parse
     assert_instance_of XML::Document, doc
 
     root = doc.root
