@@ -2,10 +2,8 @@ require "xml"
 require 'test/unit'
 
 class NodeCommentTest < Test::Unit::TestCase
-  def setup()
-    xp = XML::Parser.new()
-    str = '<root></root>'
-    assert_equal(str, xp.string = str)
+  def setup
+    xp = XML::Parser.string('<root></root>')
     @doc = xp.parse
     assert_instance_of(XML::Document, @doc)
     @root = @doc.root
