@@ -5,7 +5,7 @@ require 'test/unit'
 # [libxml-devel] Segmentation fault when add the cloned/copied node
 # 2007/11/27 20:51
 
-class TC_XML_Node_Copy < Test::Unit::TestCase
+class TestNodeCopy < Test::Unit::TestCase
   def setup
     str = <<-STR
       <html><body>
