@@ -1,12 +1,10 @@
-# $Id: tc_node_xlink.rb 481 2008-07-19 08:59:39Z cfis $
+# $Id: tc_node_xlink.rb 710 2009-01-20 05:30:51Z cfis $
 require "xml"
 require 'test/unit'
 
 class TC_XML_Node_XLink < Test::Unit::TestCase
   def setup()
-    xp = XML::Parser.new()
-    str = '<ruby_array xmlns:xlink="http://www.w3.org/1999/xlink/namespace/"><fixnum xlink:type="simple">one</fixnum></ruby_array>'
-    assert_equal(str, xp.string = str)
+    xp = XML::Parser.string('<ruby_array xmlns:xlink="http://www.w3.org/1999/xlink/namespace/"><fixnum xlink:type="simple">one</fixnum></ruby_array>')
     doc = xp.parse
     assert_instance_of(XML::Document, doc)
     @root = doc.root
