@@ -3,8 +3,8 @@ require 'test/unit'
 
 class TestRelaxNG < Test::Unit::TestCase
   def setup
-    xp = XML::Parser.new
-    @doc = XML::Document.file(File.join(File.dirname(__FILE__), 'model/shiporder.xml'))
+    file = File.join(File.dirname(__FILE__), 'model/shiporder.xml')
+    @doc = XML::Document.file(file)
   end
   
   def teardown
@@ -34,7 +34,7 @@ class TestRelaxNG < Test::Unit::TestCase
 
     assert_not_nil(error)
     assert_kind_of(XML::Error, error)
-    assert(error.message.match(/Error: Did not expect element invalid there at/))
+    assert(error.message.match(/Error: Did not expect element invalid there/))
     assert_equal(XML::Error::RELAXNGV, error.domain)
     assert_equal(XML::Error::LT_IN_ATTRIBUTE, error.code)
     assert_equal(XML::Error::ERROR, error.level)
