@@ -12,15 +12,8 @@ class TestXInclude < Test::Unit::TestCase
   end
 
   def test_ruby_xml_xinclude
-    expected = <<-EOS
-<?xml version="1.0"?>
-<document xmlns:xi="http://www.w3.org/2001/XInclude">
-  <p>This libxml2 binding has the following project information:
-   <code>This is some text to include in an xml file via XInclude.</code></p>
-</document>
-EOS
-
     assert_equal(1, @doc.xinclude)
-    assert_equal(expected, @doc.to_s)    
+    assert_equal("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<document xmlns:xi=\"http://www.w3.org/2001/XInclude\">\n  <p>This libxml2 binding has the following project information:\n   <code>This is some text to include in an xml file via XInclude.</code></p>\n</document>\n",
+                 @doc.to_s)
   end
 end
