@@ -16,7 +16,6 @@ class TestXPath < Test::Unit::TestCase
     assert_instance_of(XML::XPath::Object, nodes)
     assert_equal(1, nodes.length)
     assert_equal(nodes.xpath_type, XML::XPath::NODESET)
-    assert_instance_of(XML::XPath::Context, nodes.context)
   end
 
   def test_doc_find_first
@@ -34,14 +33,16 @@ class TestXPath < Test::Unit::TestCase
     assert_equal(3, nodes.length)
   end
 
-  def test_default_ns
-    # Pull all nodes with http://services.somewhere.com namespace
+  def test_default_ns1
+    # Find all nodes with http://services.somewhere.com namespace
     nodes = @doc.find('//*[namespace-uri()="http://services.somewhere.com"]')
     assert_equal(2, nodes.length)
     assert_equal('getManufacturerNamesResponse', nodes[0].name)
     assert_equal('IDAndNameList', nodes[1].name)
+  end
 
-    # Pull all nodes with http://services.somewhere.com namespace
+  def test_default_ns2
+    # Find all nodes with http://services.somewhere.com namespace
     nodes = @doc.find('//ns:*', 'ns:http://services.somewhere.com')
     assert_equal(2, nodes.length)
     assert_equal('getManufacturerNamesResponse', nodes[0].name)
@@ -57,6 +58,31 @@ class TestXPath < Test::Unit::TestCase
     assert_equal(3, nodes.length)
   end
 
+  def test_default_ns3
+    # Find all nodes with http://services.somewhere.com namespace
+    nodes = @doc.find('//ns:*', 'ns' => 'http://services.somewhere.com')
+    assert_equal(2, nodes.length)
+    assert_equal('getManufacturerNamesResponse', nodes[0].name)
+    assert_equal('IDAndNameList', nodes[1].name)
+  end
+
+  def test_default_ns4
+    # Find all nodes with http://services.somewhere.com namespace
+    nodes = @doc.find('//ns:*', :ns => 'http://services.somewhere.com')
+    assert_equal(2, nodes.length)
+    assert_equal('getManufacturerNamesResponse', nodes[0].name)
+    assert_equal('IDAndNameList', nodes[1].name)
+  end
+
+  def test_default_ns5
+    # Find all nodes with http://services.somewhere.com namespace
+    XML::Namespace.new(@doc.root, 'ns', 'http://services.somewhere.com')
+    nodes = @doc.find('//ns:*')
+    assert_equal(2, nodes.length)
+    assert_equal('getManufacturerNamesResponse', nodes[0].name)
+    assert_equal('IDAndNameList', nodes[1].name)
+  end
+
   def test_attribute_ns
     # Pull all nodes with http://services.somewhere.com namespace
     nodes = @doc.find('@soap:encodingStyle')
@@ -77,7 +103,7 @@ class TestXPath < Test::Unit::TestCase
     assert_not_nil(node)
 
     # Register namespace
-    doc.root.register_default_namespace('atom')
+    doc.root.namespaces.default_prefix = 'atom'
     node = doc.find("atom:title")
     assert_not_nil(node)
   end
@@ -89,6 +115,7 @@ class TestXPath < Test::Unit::TestCase
     # Since we are searching on the node, don't have to register namespace
     nodes = node.find('ns1:name')
     assert_equal(1, nodes.length)
+		assert_equal(nodes.first.object_id, nodes.last.object_id, 'First and last should be the same')
     assert_equal('name', nodes.first.name)
     assert_equal('man1', nodes.first.content)
   end
@@ -109,12 +136,6 @@ class TestXPath < Test::Unit::TestCase
     end
   end
 
-  # This causes a segmentation fault on Windows for some reason
-  #def test_nodes_debug
-   # nodes = @doc.find('//ns1:IdAndName', 'ns1:http://domain.somewhere.com')
-    #nodes.debug
-  # end
-
   def test_memory
     # This sometimes causes a segmentation fault because
     # an xml document is sometimes freed before the
@@ -141,4 +162,32 @@ class TestXPath < Test::Unit::TestCase
     GC.start
     assert_equal('Envelope', nodes.first.name)
   end
-end
\ No newline at end of file
+
+  def test_xpath_namespace_nodes
+    doc = XML::Document.string('<feed xmlns="http://www.w3.org/2005/Atom" xmlns:xhtml="http://www.w3.org/1999/xhtml"><entry/></feed>')
+    nodes = doc.find('//atom:entry|namespace::*', :atom => "http://www.w3.org/2005/Atom")
+    assert_equal(4, nodes.length)
+
+    node = nodes[0]
+    assert_equal(XML::Node::ELEMENT_NODE, node.node_type)
+
+    node = nodes[1]
+    assert_equal(XML::Node::NAMESPACE_DECL, node.node_type)
+
+    node = nodes[2]
+    assert_equal(XML::Node::NAMESPACE_DECL, node.node_type)
+
+    node = nodes[3]
+    assert_equal(XML::Node::NAMESPACE_DECL, node.node_type)
+  end
+
+	# Test to make sure we don't get nil on empty results.
+	# This is also to test that we don't segfault due to our C code getting a NULL pointer
+	# and not handling it properly.
+	def test_xpath_empty_result
+    doc = XML::Document.string('<html><body><p>Welcome to XHTML land!</p></body></html>')
+		nodes = doc.find("//object/param[translate(@name, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ', 'abcdefghijklmnopqrstuvwxyz') = 'wmode']")
+		assert_not_nil nodes
+	end
+  
+end
