@@ -1,19 +1,21 @@
 require 'tc_xml'
 require 'tc_attributes'
+require 'tc_attr'
+require 'tc_attr_decl'
 require 'tc_document'
 require 'tc_document_write'
 require 'tc_dtd'
-require 'tc_input'
 require 'tc_html_parser'
+require 'tc_namespaces'
+require 'tc_namespace'
 require 'tc_node'
-require 'tc_node_attr'
 require 'tc_node_cdata'
 require 'tc_node_comment'
 require 'tc_node_copy'
 require 'tc_node_edit'
 require 'tc_node_text'
+require 'tc_node_write'
 require 'tc_node_xlink'
-require 'tc_ns'
 require 'tc_parser'
 require 'tc_parser_context'
 require 'tc_reader'
@@ -21,7 +23,6 @@ require 'tc_relaxng'
 require 'tc_sax_parser'
 require 'tc_schema'
 require 'tc_traversal'
-require 'tc_well_formed'
 require 'tc_xinclude'
 require 'tc_xpath'
 require 'tc_xpath_expression'
